/*
Copyright (C) 2006 - 2009 Evan Teran
                   eteran@alum.rit.edu

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

#ifndef STATE_20060715_H_
#define STATE_20060715_H_

#include "Types.h"
#include "API.h"

class QString;

class EDB_EXPORT State {
public:
	State();
	~State() {}

private:
	// not copyable, use to/fromBackup routines
	State(const State &other);
	State &operator=(const State &other);
	
public:
	// what an FPU register looks like	
	struct EDB_EXPORT i387Register {
	public:
		i387Register();
		i387Register(const i387Register &other);
		i387Register &operator=(const i387Register &other);
	public:
		double toDouble() const;
	public:
		bool operator==(const i387Register &other) const;
		bool operator!=(const i387Register &other) const;
	public:
		union {
			quint8 raw[sizeof(long double)];
			long double raw_ldouble;
		};
	};

public:
	// the actual state storage
	edb::reg_t ebx;
	edb::reg_t ecx;
	edb::reg_t edx;
	edb::reg_t esi;
	edb::reg_t edi;
	edb::reg_t ebp;
	edb::reg_t eax;
	edb::reg_t esp;
	union { edb::reg_t _cs; quint16 cs; };
	union { edb::reg_t _ds; quint16 ds; };
	union { edb::reg_t _es; quint16 es; };
	union { edb::reg_t _fs; quint16 fs; };
	union { edb::reg_t _gs; quint16 gs; };
	union { edb::reg_t _ss; quint16 ss; };
	union { edb::reg_t eip; quint16 ip; };
	union { edb::reg_t eflags; quint16 flags; };
	edb::reg_t orig_eax;
	
	i387Register st[8];
	edb::reg_t dr[8];
	
	edb::address_t gs_base;
	edb::address_t fs_base;

public:
	void adjustStack(int bytes);
	void setInstructionPointer(edb::address_t value);
	edb::address_t instructionPointer() const;
	edb::address_t stackPointer() const;
	edb::address_t framePointer() const;
	void frombackup(const State &backup);
	void toBackup(State &backup) const;
	edb::reg_t value(const QString &reg, bool &ok) const;
	edb::reg_t value(const QString &reg) const;
	void clear();
	
public:
	enum {		
		// groups, these catagories should remain as portable as possible
		REG_GPR		= 0x0001,
		REG_IP		= 0x0002,
		REG_SEG		= 0x0004,
		REG_COND	= 0x0008,
		REG_FPU		= 0x0010,
		REG_ANY		= -1
	};
};

#endif
