/*
Copyright (C) 2006 - 2009 Evan Teran
                   eteran@alum.rit.edu

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

#include "HardwareBreakpoints.h"
#include "Debugger.h"
#include "DialogHWBreakpoints.h"
#include "DebuggerCoreInterface.h"
#include "State.h"
#include "Debugger.h"

#include <QMenu>
#include <QDialog>

// TODO: at the moment, nearly this entire file is x86/x86-64 specific
//       we need to figure out a proper way to support (if at all) non
//       x86 arches

//------------------------------------------------------------------------------
// Name: HardwareBreakpoints()
// Desc: 
//------------------------------------------------------------------------------
HardwareBreakpoints::HardwareBreakpoints() : m_Menu(0), m_Dialog(0), m_OldEventHandler(0) {
}

//------------------------------------------------------------------------------
// Name: menu(QWidget *parent)
// Desc: 
//------------------------------------------------------------------------------
QMenu *HardwareBreakpoints::menu(QWidget *parent) {
	if(m_Menu == 0) {
		m_Menu = new QMenu(tr("Hardware BreakpointManager"), parent);
		m_Menu->addAction(tr("&Hardware Breakpoints"), this, SLOT(doMenu()), QKeySequence(tr("Ctrl+Shift+H")));
	}
	
	return m_Menu;
}

//------------------------------------------------------------------------------
// Name: 
// Desc: 
//------------------------------------------------------------------------------
void HardwareBreakpoints::setupBP(State &state, int num, bool enabled, edb::address_t addr, int type, int size) {

	const int N1 = 16 + (num * 4);
	const int N2 = 18 + (num * 4);

	// default to disabled
	state.dr[7] = (state.dr[7] & ~(0x01 << (num * 2)));
	
	if(enabled) {
		// set the address
		state.dr[num] = addr;

		// enable this breakpoint
		state.dr[7] |= (0x01 << (num * 2));

		// setup the type
		switch(type) {
		case 2:
			// read/write
			state.dr[7] = (state.dr[7] & ~(0x03 << N1)) | (0x03 << N1);
			break;
		case 1:
			// write
			state.dr[7] = (state.dr[7] & ~(0x03 << N1)) | (0x01 << N1);
			break;
		case 0:
			// execute
			state.dr[7] = (state.dr[7] & ~(0x03 << N1));
			break;
		}

		if(type != 0) {
			// setup the size
			switch(size) {
			case 2:
				// 4 bytes
				state.dr[7] = (state.dr[7] & ~(0x03 << N2)) | (0x03 << N2);
				break;
			case 1:
				// 2 bytes
				state.dr[7] = (state.dr[7] & ~(0x03 << N2)) | (0x01 << N2);
				break;
			case 0:
				// 1 byte
				state.dr[7] = (state.dr[7] & ~(0x03 << N2));
				break;
			}
		} else {
			state.dr[7] = (state.dr[7] & ~(0x03 << N2));
		}			
	}
}

//------------------------------------------------------------------------------
// Name: 
// Desc: 
//------------------------------------------------------------------------------
void HardwareBreakpoints::setupBreakPoints(DialogHWBreakpoints *p) {
	
	// TODO: assert that we are paused
	
	const bool enabled = 
		p->ui.chkBP1->isChecked() ||
		p->ui.chkBP2->isChecked() ||
		p->ui.chkBP3->isChecked() ||
		p->ui.chkBP4->isChecked();

	if(enabled) {
		// we want to be enabled, if we aren't already hooked, 
		// hook it
		if(m_OldEventHandler == 0) {
			m_OldEventHandler = edb::v1::setDebugEventHandler(this);
		}

		State state;
		bool ok;
		edb::v1::debuggerBase->getState(state);
		
		edb::address_t addr;
		
		addr = edb::v1::stringToAddress(p->ui.txtBP1->text(), ok);
		if(ok) {	
			setupBP(state, 0, p->ui.chkBP1->isChecked(), addr, p->ui.cmbType1->currentIndex(), p->ui.cmbSize1->currentIndex());
		}
		
		addr = edb::v1::stringToAddress(p->ui.txtBP2->text(), ok);
		if(ok) {	
			setupBP(state, 1, p->ui.chkBP2->isChecked(), addr, p->ui.cmbType2->currentIndex(), p->ui.cmbSize2->currentIndex());
		}
		
		addr = edb::v1::stringToAddress(p->ui.txtBP3->text(), ok);
		if(ok) {	
			setupBP(state, 2, p->ui.chkBP3->isChecked(), addr, p->ui.cmbType3->currentIndex(), p->ui.cmbSize3->currentIndex());
		}
		
		addr = edb::v1::stringToAddress(p->ui.txtBP4->text(), ok);
		if(ok) {	
			setupBP(state, 3, p->ui.chkBP4->isChecked(), addr, p->ui.cmbType4->currentIndex(), p->ui.cmbSize4->currentIndex());
		}

		edb::v1::debuggerBase->setState(state);

	} else {
	
		State state;
		edb::v1::debuggerBase->getState(state);
		state.dr[7] = 0;
		edb::v1::debuggerBase->setState(state);
	
		// we want to be disabled and we have hooked, so unhook
		if(m_OldEventHandler != 0) {
			edb::v1::setDebugEventHandler(m_OldEventHandler);
			m_OldEventHandler = 0;
		}			
	}
}

//------------------------------------------------------------------------------
// Name: doMenu()
// Desc: 
//------------------------------------------------------------------------------
void HardwareBreakpoints::doMenu() {

	if(m_Dialog == 0) {
		m_Dialog = new DialogHWBreakpoints(edb::v1::debuggerUI);
	}
	
	DialogHWBreakpoints *const p = qobject_cast<DialogHWBreakpoints *>(m_Dialog);
	
	if(p != 0) {	
		if(m_Dialog->exec() == QDialog::Accepted) {
			setupBreakPoints(p);
		}
	}
}

//------------------------------------------------------------------------------
// Name: handleEvent(const DebugEvent &event)
// Desc: this hooks the debug event handler so we can make the breakpoints
//       able to be resumed
//------------------------------------------------------------------------------
void HardwareBreakpoints::handleEvent(const DebugEvent &event) {

	if(event.reason() == DebugEvent::EVENT_STOPPED) {
		if(event.stopCode() == DebugEvent::sigtrap) {
			// check DR6 to see if it was a HW BP event
			// if so, set the resume flag
			State state;
			edb::v1::debuggerBase->getState(state);
			
			if((state.dr[6] & 0x0f) != 0x00) {
#if defined(EDB_X86_64)
				state.rflags |= (1 << 16);
#elif defined(EDB_X86)
				state.eflags |= (1 << 16);
#else
#error "Unsupported Platform"
#endif
				edb::v1::debuggerBase->setState(state);
			}
		}
	}
	
	m_OldEventHandler->handleEvent(event);
}


Q_EXPORT_PLUGIN2(HardwareBreakpoints, HardwareBreakpoints)
