/*
Copyright (C) 2006 - 2009 Evan Teran
                   eteran@alum.rit.edu

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

#include "DialogReferences.h"
#include "DebuggerCoreInterface.h"
#include "Debugger.h"
#include "Util.h"
#include "MemoryRegions.h"
#include "Debugger.h"

#include <QVector>
#include <QMessageBox>

//------------------------------------------------------------------------------
// Name: DialogReferences(QWidget *parent)
// Desc: constructor
//------------------------------------------------------------------------------
DialogReferences::DialogReferences(QWidget *parent) : QDialog(parent) {
	ui.setupUi(this);
	connect(this, SIGNAL(updateProgress(int)), ui.progressBar, SLOT(setValue(int)));
}

//------------------------------------------------------------------------------
// Name: showEvent(QShowEvent *)
// Desc: 
//------------------------------------------------------------------------------
void DialogReferences::showEvent(QShowEvent *) {
	ui.listWidget->clear();
	ui.progressBar->setValue(0);
}

//------------------------------------------------------------------------------
// Name: doFind()
// Desc: 
//------------------------------------------------------------------------------
void DialogReferences::doFind() {
	bool ok;
	const edb::address_t address	= edb::v1::stringToAddress(ui.txtAddress->text(), ok);
	const edb::address_t pageSize	= edb::v1::debuggerBase->pageSize();
	
	if(ok) {
		edb::v1::memoryRegions().sync();
		const QVector<MemRegion> regions = edb::v1::memoryRegions().regions();

		int i = 0;
		Q_FOREACH(const MemRegion &region, regions) {
			// a short circut for speading things up
			if(region.accessible() || !ui.chkSkipNoAccess->isChecked()) {

				const edb::address_t size_in_pages = region.size() / pageSize;
				
				try {
				
					QVector<quint8> pages(size_in_pages * pageSize);
					const quint8 *const pages_end = &pages[0] + size_in_pages * pageSize;

					if(edb::v1::debuggerBase->readPages(region.start, &pages[0], size_in_pages)) {
						const quint8 *p = &pages[0];
						while(p != pages_end) {

							if(static_cast<std::size_t>(pages_end - p) < sizeof(edb::address_t)) {
								break;
							}

							if(*reinterpret_cast<const edb::address_t *>(p) == address) {
								const edb::address_t addr = p - &pages[0] + region.start;
								ui.listWidget->addItem(edb::v1::formatPointer(addr));
							}

							emit updateProgress(util::percentage(i, regions.size(), p - &pages[0], region.size()));
							++p;
						}
					}
				} catch(const std::bad_alloc &) {
					QMessageBox::information(0, tr("Memroy Allocation Error"),
						tr("Unable to satisfy memory allocation request for requested region."));
				}
			} else {
				emit updateProgress(util::percentage(i, regions.size()));
			}
			++i;
		}
	}
}

//------------------------------------------------------------------------------
// Name: on_btnFind_clicked()
// Desc: find button event handler
//------------------------------------------------------------------------------
void DialogReferences::on_btnFind_clicked() {
	ui.btnFind->setEnabled(false);
	ui.listWidget->clear();
	ui.progressBar->setValue(0);
	doFind();
	ui.progressBar->setValue(100);
	ui.btnFind->setEnabled(true);
}

//------------------------------------------------------------------------------
// Name: on_listWidget_itemDoubleClicked(QListWidgetItem *item)
// Desc: follows the foudn item in the data view
//------------------------------------------------------------------------------
void DialogReferences::on_listWidget_itemDoubleClicked(QListWidgetItem *item) {
	bool ok;
	const edb::address_t addr = edb::v1::stringToAddress(item->text(), ok);
	if(ok) {
		edb::v1::dumpData(addr, false);
	}
}
