/*
Copyright (C) 2006 - 2009 Evan Teran
                   eteran@alum.rit.edu

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

#include "ByteShiftArray.h"

//------------------------------------------------------------------------------
// Name: ByteShiftArray(std::size_t size)
// Desc: constructor
//------------------------------------------------------------------------------
ByteShiftArray::ByteShiftArray(std::size_t size) : m_Data(size) {
}

//------------------------------------------------------------------------------
// Name: ByteShiftArray(const ByteShiftArray& other)
// Desc: copy constructor
//------------------------------------------------------------------------------
ByteShiftArray::ByteShiftArray(const ByteShiftArray& other) : m_Data(other.m_Data) {
}

//------------------------------------------------------------------------------
// Name: operator=(const ByteShiftArray &other
// Desc: assignment operator
//------------------------------------------------------------------------------
ByteShiftArray &ByteShiftArray::operator=(const ByteShiftArray &other) {
	if(&other != this) {
		ByteShiftArray(other).swap(*this);
	}
	return *this;
}

//------------------------------------------------------------------------------
// Name: 
// Desc: 
//------------------------------------------------------------------------------
void ByteShiftArray::swap(ByteShiftArray &other) {
	qSwap(m_Data, other.m_Data);
}

//------------------------------------------------------------------------------
// Name: shl()
// Desc: shifts data left one byte and shifts in a 0
//------------------------------------------------------------------------------
ByteShiftArray &ByteShiftArray::shl() {
	for(int i = 1; i < m_Data.size(); ++i) {
		m_Data[i - 1] = m_Data[i];
	}
	m_Data.back() = 0;
	return *this;
}

//------------------------------------------------------------------------------
// Name: shr()
// Desc: shifts data right one byte and shifts in a 0
//------------------------------------------------------------------------------
ByteShiftArray &ByteShiftArray::shr() {
	for(int i = 0; i < m_Data.size() - 1; ++i) {
		m_Data[i + 1] = m_Data[i];
	}
	m_Data.first() = 0;
	return *this;
}

//------------------------------------------------------------------------------
// Name: size() const
// Desc: returns size of this byte array
//------------------------------------------------------------------------------
std::size_t ByteShiftArray::size() const {
	return m_Data.size();
}

//------------------------------------------------------------------------------
// Name: operator[](std::size_t i)
// Desc: returns and l-value version of an element in the byte array
//------------------------------------------------------------------------------
quint8 &ByteShiftArray::operator[](std::size_t i) {
	return m_Data[i];
}

//------------------------------------------------------------------------------
// Name: operator[](std::size_t i)
// Desc: returns and r-value version of an element in the byte array
//------------------------------------------------------------------------------
quint8 ByteShiftArray::operator[](std::size_t i) const {
	return m_Data[i];
}

//------------------------------------------------------------------------------
// Name: data() const
// Desc: returns a read only pointer to the data this byte array holds
//------------------------------------------------------------------------------
const quint8 *ByteShiftArray::data() const {
	return &m_Data[0];
}

//------------------------------------------------------------------------------
// Name: clear()
// Desc: zeros out the byte array
//------------------------------------------------------------------------------
void ByteShiftArray::clear() {
	m_Data.fill(0);
}

//------------------------------------------------------------------------------
// Name: operator<<(quint8 x)
// Desc: 
//------------------------------------------------------------------------------
ByteShiftArray &ByteShiftArray::operator<<(quint8 x) {
	shl();
	m_Data.back() = x;
	return *this;
}
