/*
Copyright (C) 2006 - 2009 Evan Teran
                   eteran@alum.rit.edu

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

#include "State.h"
#include "Util.h"
#include <QString>
#include <QtAlgorithms>
#include <cstring>

//------------------------------------------------------------------------------
// Name: 
// Desc: 
//------------------------------------------------------------------------------
State::i387Register::i387Register() {
	std::memset(raw, 0, sizeof(raw));
}

//------------------------------------------------------------------------------
// Name: 
// Desc: 
//------------------------------------------------------------------------------
State::i387Register::i387Register(const i387Register &other) {
	std::memcpy(raw, other.raw, sizeof(raw));
}

//------------------------------------------------------------------------------
// Name: 
// Desc: 
//------------------------------------------------------------------------------
State::i387Register &State::i387Register::operator=(const State::i387Register &other) {
	if(this != &other) {
		std::memcpy(raw, other.raw, sizeof(raw));
	}
	
	return *this;
}

//------------------------------------------------------------------------------
// Name: 
// Desc: 
//------------------------------------------------------------------------------
double State::i387Register::toDouble() const {
	return static_cast<double>(raw_ldouble);
}

//------------------------------------------------------------------------------
// Name: 
// Desc: 
//------------------------------------------------------------------------------
bool State::i387Register::operator==(const State::i387Register &other) const {
	return std::memcmp(raw, other.raw, sizeof(raw)) == 0;
}

//------------------------------------------------------------------------------
// Name: 
// Desc: 
//------------------------------------------------------------------------------
bool State::i387Register::operator!=(const State::i387Register &other) const {
	return !(*this == other);
}

//------------------------------------------------------------------------------
// Name: State()
// Desc: constructor
//------------------------------------------------------------------------------
State::State() {
	clear();
}

//------------------------------------------------------------------------------
// Name: 
// Desc: 
//------------------------------------------------------------------------------
void State::frombackup(const State &backup) {
	ebx = backup.ebx;
	ecx = backup.ecx;
	edx = backup.edx;
	esi = backup.esi;
	edi = backup.edi;
	ebp = backup.ebp;
	eax = backup.eax;
	esp = backup.esp;
	_cs = backup._cs;
	_ds = backup._ds;
	_es = backup._es;
	_fs = backup._fs;
	_gs = backup._gs;
	_ss = backup._ss;
	eip = backup.eip;
	eflags = backup.eflags;
	orig_eax = backup.orig_eax;
	gs_base = backup.gs_base;
	fs_base = backup.fs_base;
	
	qCopy(backup.st, backup.st + 8, st);
}

//------------------------------------------------------------------------------
// Name: 
// Desc: 
//------------------------------------------------------------------------------
void State::toBackup(State &backup) const {
	backup.ebx = ebx;
	backup.ecx = ecx;
	backup.edx = edx;
	backup.esi = esi;
	backup.edi = edi;
	backup.ebp = ebp;
	backup.eax = eax;
	backup.esp = esp;
	backup._cs = _cs;
	backup._ds = _ds;
	backup._es = _es;
	backup._fs = _fs;
	backup._gs = _gs;
	backup._ss = _ss;
	backup.eip = eip;
	backup.eflags = eflags;
	backup.orig_eax = orig_eax;
	backup.gs_base = gs_base;
	backup.fs_base = fs_base;
	
	qCopy(st, st + 8, backup.st);
}

//------------------------------------------------------------------------------
// Name: clear()
// Desc: 
//------------------------------------------------------------------------------
void State::clear() {
	eax = 0;
	ebx = 0;
	ecx = 0;
	edx = 0;
	esi = 0;
	edi = 0;
	ebp = 0;
	esp = 0;
	_cs = 0;
	_ds = 0;
	_es = 0;
	_fs = 0;
	_gs = 0;
	_ss = 0;
	eip = 0;
	eflags = 0;
	orig_eax = 0;
	
	std::memset(st, 0, sizeof(st));
	std::memset(dr, 0, sizeof(dr));
	
	gs_base = 0;
	fs_base = 0;
}

//------------------------------------------------------------------------------
// Name: 
// Desc: 
//------------------------------------------------------------------------------
edb::address_t State::instructionPointer() const {
	return eip;
}

//------------------------------------------------------------------------------
// Name: 
// Desc: 
//------------------------------------------------------------------------------
edb::address_t State::stackPointer() const {
	return esp;
}

//------------------------------------------------------------------------------
// Name: 
// Desc: 
//------------------------------------------------------------------------------
edb::address_t State::framePointer() const {
	return ebp;
}

//------------------------------------------------------------------------------
// Name: 
// Desc: 
//------------------------------------------------------------------------------
edb::reg_t State::value(const QString &reg) const {
	bool ok;
	return value(reg, ok);
}

//------------------------------------------------------------------------------
// Name: value(const QString &reg, bool *ok)
// Desc: a function to return the value of a register based on it's name
//------------------------------------------------------------------------------
edb::reg_t State::value(const QString &reg, bool &ok) const {
	edb::reg_t ret = 0;
	ok = false;

	const QString lreg = reg.toLower();

	if(lreg == "eax") { ret = eax; ok = true; }
	else if(lreg == "ebx") { ret = ebx; ok = true; }
	else if(lreg == "ecx") { ret = ecx; ok = true; }
	else if(lreg == "edx") { ret = edx; ok = true; }
	else if(lreg == "ebp") { ret = ebp; ok = true; }
	else if(lreg == "esp") { ret = esp; ok = true; }
	else if(lreg == "esi") { ret = esi; ok = true; }
	else if(lreg == "edi") { ret = edi; ok = true; }
	else if(lreg == "eip") { ret = eip; ok = true; }
	else if(lreg == "ax") { ret = eax & 0xffff; ok = true; }
	else if(lreg == "bx") { ret = ebx & 0xffff; ok = true; }
	else if(lreg == "cx") { ret = ecx & 0xffff; ok = true; }
	else if(lreg == "dx") { ret = edx & 0xffff; ok = true; }
	else if(lreg == "bp") { ret = ebp & 0xffff; ok = true; }
	else if(lreg == "sp") { ret = esp & 0xffff; ok = true; }
	else if(lreg == "si") { ret = esi & 0xffff; ok = true; }
	else if(lreg == "di") { ret = edi & 0xffff; ok = true; }
	else if(lreg == "al") { ret = eax & 0xff; ok = true; }
	else if(lreg == "bl") { ret = ebx & 0xff; ok = true; }
	else if(lreg == "cl") { ret = ecx & 0xff; ok = true; }
	else if(lreg == "dl") { ret = edx & 0xff; ok = true; }
	else if(lreg == "ah") { ret = (eax >> 8) & 0xff; ok = true; }
	else if(lreg == "bh") { ret = (ebx >> 8) & 0xff; ok = true; }
	else if(lreg == "ch") { ret = (ecx >> 8) & 0xff; ok = true; }
	else if(lreg == "dh") { ret = (edx >> 8) & 0xff; ok = true; }
	else if(lreg == "cs") { ret = 0; ok = true; }
	else if(lreg == "ds") { ret = 0; ok = true; }
	else if(lreg == "es") { ret = 0; ok = true; }
	else if(lreg == "fs") { ret = fs_base; ok = true; }
	else if(lreg == "gs") { ret = gs_base; ok = true; }
	else if(lreg == "ss") { ret = 0; ok = true; }
	
	return ret;
}

//------------------------------------------------------------------------------
// Name: 
// Desc: 
//------------------------------------------------------------------------------
void State::setInstructionPointer(edb::address_t value) {
	eip = value;
}

//------------------------------------------------------------------------------
// Name: 
// Desc: 
//------------------------------------------------------------------------------
void State::adjustStack(int bytes) {
	esp += bytes;
}
