/*
Copyright (C) 2006 - 2009 Evan Teran
                   eteran@alum.rit.edu

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

#include "QCategoryList.h"
#include "RegisterViewDelegate.h"
#include <QHeaderView>
#include <QTreeView>
#include <QMouseEvent>

//------------------------------------------------------------------------------
// Name: QCategoryList(QWidget *parent)
// Desc: 
//------------------------------------------------------------------------------
QCategoryList::QCategoryList(QWidget *parent) : QTreeWidget(parent) {

	// set the delagate
	setItemDelegate(new RegisterViewDelegate(this, this));
	
	// hide the exapander, since we provide our own
	setRootIsDecorated(false);
	
	setColumnCount(1);
	header()->hide();
	header()->setResizeMode(QHeaderView::Stretch);
	
	connect(this, SIGNAL(itemPressed(QTreeWidgetItem *, int)), SLOT(handleMousePress(QTreeWidgetItem*)));
}

//------------------------------------------------------------------------------
// Name: 
// Desc: 
//------------------------------------------------------------------------------
QCategoryList::~QCategoryList() {

}

//------------------------------------------------------------------------------
// Name: handleMousePress(QTreeWidgetItem *item)
// Desc: 
//------------------------------------------------------------------------------
void QCategoryList::handleMousePress(QTreeWidgetItem *item) {
	if(isCatagory(item)) {
	    setItemExpanded(item, !isItemExpanded(item));
	}
}

//------------------------------------------------------------------------------
// Name: 
// Desc: 
//------------------------------------------------------------------------------
void QCategoryList::mouseDoubleClickEvent(QMouseEvent * event) {
	
	QTreeWidgetItem *const p = itemAt(event->pos());
	if(p != 0) {
		emit itemDoubleClicked(p, 0);
	}
}

//------------------------------------------------------------------------------
// Name: addCatagory(const QString &name)
// Desc: 
//------------------------------------------------------------------------------
QTreeWidgetItem *QCategoryList::addCatagory(const QString &name) {
	QTreeWidgetItem *const cat = new QTreeWidgetItem(this);
	cat->setText(0, name);
	setItemExpanded(cat, true);
	return cat;
}

//------------------------------------------------------------------------------
// Name: addItem(QTreeWidgetItem *catagory, const QString &value)
// Desc: 
//------------------------------------------------------------------------------
QTreeWidgetItem *QCategoryList::addItem(QTreeWidgetItem *catagory, const QString &value) {
	QTreeWidgetItem *const item = new QTreeWidgetItem(catagory);
	item->setText(0, value);
	return item;
}

//------------------------------------------------------------------------------
// Name: isCatagory(QTreeWidgetItem *item) const
// Desc: 
//------------------------------------------------------------------------------
bool QCategoryList::isCatagory(QTreeWidgetItem *item) const {
	return item != 0 && item->parent() == 0;
}
