/*
Copyright (C) 2006 - 2009 Evan Teran
                   eteran@alum.rit.edu

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

#ifndef QDISASSEMBLYVIEW_20061101_H_
#define QDISASSEMBLYVIEW_20061101_H_

#include <QAbstractScrollArea>
#include <QSet>
#include <QPixmap>

#include "MemoryRegions.h"
#include "Types.h"

class AnalyzerInterface;
class QPainter;
class QTextDocument;
class SyntaxHighlighter;

class QDisassemblyView : public QAbstractScrollArea {
	Q_OBJECT
		
public:
	QDisassemblyView(QWidget *parent = 0);
	~QDisassemblyView();
	
protected:
	virtual void paintEvent(QPaintEvent *event);
	virtual void mouseDoubleClickEvent(QMouseEvent *event);
	virtual void mousePressEvent(QMouseEvent *event);
	virtual void mouseMoveEvent(QMouseEvent *event);
	virtual void mouseReleaseEvent(QMouseEvent *event);
	virtual bool event(QEvent *event);

public:
	bool addressShown(edb::address_t address) const;
	edb::address_t addressFromPoint(const QPoint &pos) const;
	edb::address_t selectedAddress() const;
	int selectedSize() const;

public Q_SLOTS:
	void setFont(const QFont &f);
	void resizeEvent(QResizeEvent *event);
	void scrollTo(edb::address_t address);
	void setAddressOffset(edb::address_t address);
	void setRegion(const MemRegion &r);
	void setCurrentAddress(edb::address_t address);
	void clear();
	void repaint();
	void setShowAddressSeparator(bool value);
	
signals:
	void breakPointToggled(edb::address_t address);

private:
	static int InstructionSize(quint8 *buffer, std::size_t size);
	static bool nearLine(int x, int linex);

private:
	int lineHeight() const;
	int getInstructionSize(edb::address_t address, bool &ok) const;
	int getInstructionSize(edb::address_t address, bool &ok, quint8 *buf, int &size) const;
	void updateScrollbars();
	void updateSelectedAddress(QMouseEvent *event);
	int line1() const;
	int line2() const;
	int autoLine1() const;
	int addressLen() const;
	QString formatAddress(edb::address_t address) const;
	QString formatInstructionBytes(const edb::Instruction &insn, int maxStringPx, const QFontMetrics &metrics) const;
	QString formatInstructionBytes(const edb::Instruction &insn) const;
	edb::address_t addressFromCoord(int x, int y) const;
	int drawInstruction(QPainter &painter, const edb::Instruction &insn, bool upper, int y, int line_height, int l2) const;

private:	
	void drawFunctionMarkers(QPainter &painter, edb::address_t address, int l2, int y, int insn_size, AnalyzerInterface *analyzer);
	
private:
	edb::address_t			m_AddressOffset;
	edb::address_t	 		m_CurrentAddress;
	int						m_FontWidth;	// width of a character in this font
	int						m_FontHeight;	// height of a character in this font
	QPixmap					m_Breakpoint;
	QPixmap					m_CurrentAddressIcon;
	bool					m_ShowAddressSeparator;
	MemRegion				m_Region;
	char					m_AddressFormatString[32];
	edb::address_t			m_SelectedInstructionAddress;
	int						m_SelectedInstructionSize;
	QSet<edb::address_t>	m_ShowAddresses;
	bool					m_MovingLine1;
	bool					m_MovingLine2;
	int						m_Line1;
	int						m_Line2;
	
	SyntaxHighlighter		*m_Highhighlighter;
};

#endif
